// Copyright 2008-2009 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// <http://www.omnigroup.com/developer/sourcecode/sourcelicense/>.

#import "OATargetSelection.h"

RCS_ID("$Header: $")

@implementation NSObject (OATargetSelection)

- (BOOL)applyToResponderChain:(OAResponderChainApplier)applier;
{
    return applier(self);
}

- (id)responsibleTargetForAction:(SEL)action sender:(id)sender;
{
    //NSLog(@"   ... can has %@ handle %@ from %@?", [self shortDescription], NSStringFromSelector(action), [sender shortDescription]);
    
    if (![self respondsToSelector:action])
        return nil;
    
    if ([sender isKindOfClass:[NSMenuItem class]]) {
        if ([self respondsToSelector:@selector(validateMenuItem:)] && ![self validateMenuItem:sender])
            return nil;
    } else if ([sender isKindOfClass:[NSToolbarItem class]]) {
        if ([self respondsToSelector:@selector(validateToolbarItem:)] && ![self validateToolbarItem:sender])
            return nil;
    } else if ([sender conformsToProtocol:@protocol(NSValidatedUserInterfaceItem)]) {
        if ([self respondsToSelector:@selector(validateUserInterfaceItem:)]) {
            OBASSERT([self conformsToProtocol:@protocol(NSUserInterfaceValidations)]); // or should we check for conformance...
            if (![(id <NSUserInterfaceValidations>)self validateUserInterfaceItem:sender])
                return nil;
        }
    }
    
    //NSLog(@"%@ is responsible", [self shortDescription]);
    
    return self;
}

@end

@interface NSResponder (OATargetSelection)
@end
@implementation NSResponder (OATargetSelection)

- (BOOL)applyToResponderChain:(OAResponderChainApplier)applier;
{
    if (![super applyToResponderChain:applier])
        return NO;
    
    NSResponder *next = self.nextResponder;
    if (next && ![next applyToResponderChain:applier])
        return NO;
    
    return YES;
}

@end

@interface NSWindow (OATargetSelection)
@end
@implementation NSWindow (OATargetSelection)

- (BOOL)applyToResponderChain:(OAResponderChainApplier)applier;
{
    if (![super applyToResponderChain:applier])
        return NO;
    
    // the documentation says that the NSWindowController is consulted first.
    // file:///Developer/Documentation/DocSets/com.apple.adc.documentation.AppleSnowLeopard.CoreReference.docset/Contents/Resources/Documents/documentation/Cocoa/Conceptual/EventOverview/EventArchitecture/EventArchitecture.html
    
    NSWindowController *windowController = self.windowController;
    if (windowController && ![windowController applyToResponderChain:applier])
        return NO;
    
    id delegate = (id)self.delegate;
    if (delegate && delegate != windowController && ![delegate applyToResponderChain:applier])
        return NO;
    
    // We do the document here instead of in -[NSWindowController applyToResponderChain:] since the documentation says that the ordering is:
    //
    //   windowController
    //   window.delegate
    //   document, if different from the delegate
    //
    // If we put this in -[NSWindowController applyToResponderChain:], we'd have the document before the delegate.
    //
    // NSWindowController, inheriting from NSResponder, will just then get the default implementation (which will consider its nextResponder, an issue which isn't addressed in the documentation)do
    
    NSDocument *document = windowController.document;
    if (document && document != delegate && ![document applyToResponderChain:applier])
        return NO;
    
    return YES;
}

@end
